package com.unionpay.acp.demo;


import java.io.BufferedReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.Gson;
import com.unionpay.acp.sdk.AcpService;
import com.unionpay.acp.sdk.LogUtil;
import com.unionpay.acp.sdk.SDKConstants;
import com.unionpay.acp.sdk.SDKUtil;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

/**
 * Important: Be sure to read the comments carefully before conducting an alignment test!
 * 
 * Product:UPOP<br>
 * Function: To give examples of how to receive and process foreground notifications <br>
 * Date:  2015-09<br>
 * Version:  1.0.0 
 * Copyright:  China UnionPay<br>
 * Note: The following codes are only example codes provided to facilitate clients to conduct tests, and each client can compile these codes according to the technical documents as required. These codes are for reference only.<br>
 * Transaction description: 	This is an example page which is displayed when you click the Return to Client button after you have made the payment successfully.
 * 			For the purpose of security, you are recommended to query the interface to confirm that the transaction is successful after receiving a notification for a transaction with capital involved. For a transaction with no capital involved, you can determine that it is successful if the respCode of the notification interface is 00.
 * 			In case no notification is received, you can query the time point when the interface is called by referring to this FAQ.https://open.unionpay.com/ajweb/help/faq/list?id=77&level=0&from=0
 */
@Controller
@EnableAutoConfiguration
@RequestMapping("/SecurepayAsynDemo")
public class FrontRcvResponse extends HttpServlet {

	private static final long serialVersionUID = -4826029673018921502L;

	@Override
	public void init() throws ServletException {
		/**
		 * Initialize the parameters related to requesting UnionPay access address as well as obtaining certificate files and certificate paths to the SDKConfig class.
		 * Be sure to load such parameters each time you run them in manner of java main.
		 * In case of Web application development, this method can be written into cache by means of monitoring, and does not need to appear here.
		 */
		//Here, the method for loading a property file has been moved to web/AutoLoadServlet.java.
		//SDKConfig.getConfig().loadPropertiesFromSrc(); //Load the acp_sdk.properties file from classpath
		super.init();
	}

	@RequestMapping(value="/response_page", method= RequestMethod.POST)
	public ModelAndView response_page(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		LogUtil.writeLog("FrontRcvResponse: The foreground receives a message and returns “Start”.");
		String pageResult = "/gbk_result.jsp";

		StringBuffer buffer = new StringBuffer();
		String line = null;
		try {
			BufferedReader reader = req.getReader();
			while ((line = reader.readLine()) != null)
				buffer.append(line);
		} catch (Exception e) {
			e.printStackTrace();
		}

		String body_str =  buffer.toString();
		Gson gson = new Gson();
		LogUtil.writeLog("In the response, data: "+ gson.toJson(body_str) );

		Map<String, Object> valideData = gson.fromJson(body_str,Map.class);
		StringBuffer page = new StringBuffer();
		if (null != valideData && !valideData.isEmpty()) {
			Iterator<Entry<String, Object>> it = valideData.entrySet()
					.iterator();
			valideData = new HashMap<String, Object>(valideData.size());
			while (it.hasNext()) {
				Entry<String, Object> e = it.next();
				String key = (String) e.getKey();
				String value = String.valueOf(e.getValue());
				
				page.append("<tr><td width=\"30%\" align=\"right\">" + key
						+ "(" + key + ")</td><td>" + value + "</td></tr>");
			}
		}
		page.append("Plain text customerInfo: "+valideData + "<br>");
		req.setAttribute("result", page.toString());

		LogUtil.writeLog("FrontRcvResponse: The foreground receives a message and returns “End”.");
		return new ModelAndView("redirect:" + pageResult);

	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		//Get the address of the foreground notification in case of provisioning failure.
		resp.getWriter().write("Provisioning failure");
	}

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		//Get the address of the foreground notification in case of provisioning failure.
		resp.getWriter().write("Provisioning failure");
	}

	/**
	 * Obtain all the information in the requested parameters
	 * 
	 * @param request
	 * @return
	 */
	public static Map<String, String> getAllRequestParam(
			final HttpServletRequest request) {
		Map<String, String> res = new HashMap<String, String>();
		Enumeration<?> temp = request.getParameterNames();
		if (null != temp) {
			while (temp.hasMoreElements()) {
				String en = (String) temp.nextElement();
				String value = request.getParameter(en);
				res.put(en, value);
				// When sending messages, do not send the content below if the value of the field is null: What should be done next is deleting this field if you determine that its value is null when obtaining all parameter data.
				if (res.get(en) == null || "".equals(res.get(en))) {
					// System.out.println("======A null field name===="+en);
					res.remove(en);
				}
			}
		}
		return res;
	}

}
