package com.unionpay.acp.demo.Inquiry;


import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.Gson;
import com.unionpay.acp.sdk.*;
import com.unionpay.acp.demo.DemoBase;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

/**
 * 
 * Transaction state inquiry transaction
 * 
 *
 */
@Controller
@EnableAutoConfiguration
@RequestMapping("/SecurepayAsynDemo")
public class Form_5_2_2_Query extends HttpServlet {

	protected Gson gson = new Gson();

	@Value("${server.requestBackUrl}")
	private String requestBackUrl;

	@Value("${merchant.commonKey.CAD}")
	private String commonKey_CAD;

	@Value("${merchant.commonKey.USD}")
	private String commonKey_USD;

	@Value("${server.ott_merchantid.CAD}")
	String ott_merchantid_CAD;

	@Value("${server.ott_merchantid.USD}")
	String ott_merchantid_USD;

	@Override
	public void init(ServletConfig config) throws ServletException {
		/**
		 * Initialize the parameters related to requesting UnionPay access address as well as obtaining certificate files and certificate paths to the SDKConfig class.
		 * Be sure to load such parameters each time you run them in manner of java main.
		 * In case of Web application development, this method can be written into cache by means of monitoring, and does not need to appear here.
		 */
		//Here, the method for loading a property file has been moved to web/AutoLoadServlet.java.
		//SDKConfig.getConfig().loadPropertiesFromSrc(); //Load the acp_sdk.properties file from classpath
		super.init();
	}
	
	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		
		String merId = req.getParameter("merId");
		String IIN = req.getParameter("IIN");
		String orderId = req.getParameter("orderId");
		String txnTime = req.getParameter("txnTime");
		
		Map<String, String> data = new HashMap<String, String>();
		
		/***For an all-channel UnionPay system, all parameters can be left unchanged except the one encoding, which you need to set as required.***/
		data.put("version", DemoBase.version);                 //M
		data.put("encoding", DemoBase.encoding);               //M
		data.put("signMethod", SDKConfig.getConfig().getSignMethod()); //M
		// Value: 00
		data.put("txnType", "00");// M
		// 00: PAN query (default)
		// 01: Mobile number query (Merchant direct access mode)
		// 02: Merchant order number query
		data.put("txnSubType", "00");// M
		// Product type
		data.put("bizType", "000201");// M
		// 0: Merchant direct access
		// 1: Acquirer Access
		data.put("accessType", "1");// M
		
		// Acquirer Code (Applied if access type is acquirer access)
		data.put("acqInsCode", IIN);//C
		// Merchant Category (Applied if access type is acquirer access)
		data.put("merCatCode", "4511");// C
		// Merchant Name (Applied if access type is acquirer access)
		data.put("merName", "UPI Test Merchant");// C
		// Merchant Abbreviation (Applied if access type is acquirer access)
		data.put("merAbbr", "UPI Test");// C
		
		data.put("merId", merId);                  			   //M client number: Please modify it to the client number you have applied or the test client number 777 you have registered at open.unionpay.com.
		
		/***要调通交易以下字段必须修改***/
		data.put("orderId", orderId);                 //M**** client order number: It should be modified to the order number of the transaction queried for each transaction test.
		data.put("txnTime", txnTime);                 //M**** order delivery time: It should be modified to the order delivery time of the transaction queried for each transaction test.

		/**All request parameters have been set successfully. Now, let's sign the requested parameters, and send http.post requests and receive synchronous response messages.------------->**/
		
		Map<String, String> reqData = AcpService.sign(data,DemoBase.encoding);           //In a message, the values of certId and signature are obtained from the signData method and are assigned with values automatically. Therefore, you just need to ensure that the certificate is correctly configured.
		String url = SDKConfig.getConfig().getSingleQueryUrl();						          //At the url of the transaction request, you can read the acpsdk.singleQueryUrl in the corresponding property file acp_sdk.properties from the configuration file.
		Map<String, String> rspData = AcpService.post(reqData,url,DemoBase.encoding);     //Send request messages and receive synchronous responses (the default connection timeout is 30s and the timeout for reading the returned result is 30s); Here, after calling signData, do not make any modification to the value of any key in submitFromData before calling submitUrl. Any such modification may cause failure to signature authentication.
		
		/**To proceed the response codes, you need to compile a program based on your business logics. The logics for response code processing below are for reference only.------------->**/
		//For response code specifications, refer to Part 5 “Appendix” in Specifications for Platform Access Interfaces- by selecting Help Center > Download > Product Interface Specifications at open.unionpay.com.
		if(!rspData.isEmpty()){
			if(AcpService.validate(rspData, DemoBase.encoding)){
				LogUtil.writeLog("Signature authentication succeeds");
				if(("00").equals(rspData.get("respCode"))){//If the transaction query succeeds,
					String origRespCode = rspData.get("origRespCode");
					if(("00").equals(origRespCode)){
						//Update the state of the client order if the transaction succeeds.
						//TODO
					}else if(("03").equals(origRespCode)||
							 ("04").equals(origRespCode)||
							 ("05").equals(origRespCode)){
						//The order is being processed or the transaction state is unknown. In this case, you need to initiate a transaction state query later. [If you cannot determine whether the transaction is successful at the end, please use the transaction state in the reconciliation file.]
						//TODO
					}else{
						//Other response codes are transaction failure.
						//TODO
					}
				}else if(("34").equals(rspData.get("respCode"))){
					//The order does not exist. In this case, you can initiate a transaction state query later, or use the transaction state in the reconciliation file.
					
				}else{//If the transaction query fails and the response code is 10/11, you need to check whether the message is correct.
					//TODO
				}
			}else{
				LogUtil.writeErrorLog("Signature authentication fails");
				//TODO Find the reason why the signature authentication fails
			}
		}else{
			//The returned http state code is incorrect.
			LogUtil.writeErrorLog("No returned message is obtained or the returned http state code is not 200.");
		}
			
		String reqMessage = DemoBase.genHtmlResult(reqData);
		String rspMessage = DemoBase.genHtmlResult(rspData);
		resp.getWriter().write("Transaction state query Request message: <br/>"+reqMessage+"<br/>" + "Response message:</br>"+rspMessage+"");
	}
	
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {
		doPost(req, resp);
	}

	@RequestMapping("/secure_querypay")
	protected void secure_querypay(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		//String merId = req.getParameter("merId").equals("ON00000097")? ott_merchantid_CAD:ott_merchantid_USD;
		//String commonKey = req.getParameter("merId").equals("ON00000097")? commonKey_CAD:commonKey_USD;
		String merId = req.getParameter("merId").equals("ON00000001")? ott_merchantid_CAD:ott_merchantid_USD;
		String commonKey = req.getParameter("merId").equals("ON00000001")? commonKey_CAD:commonKey_USD;
		String orderId = req.getParameter("orderId");
		String txnTime = req.getParameter("txnTime");
		String bizType = req.getParameter("bizType");
		String cc_channelType = req.getParameter("cc_channelType");

		Map<String, Object> map = new HashMap<>();
		map.put("orderId", orderId);
		map.put("merchant_id", merId);
		map.put("txnTime", txnTime);
		map.put("channelType", "UPAYONLINE");
		map.put("bizType", bizType);
		map.put("cc_channelType", cc_channelType);

		try{
			//Send request messages and receive synchronous responses (the default connection timeout is 30s and the timeout for reading the returned result is 30s); Here, after calling signData, do not make any modification to the value of any key in submitFromData before calling submitUrl. Any such modification may cause failure to signature authentication.
			String md5 = AppCommonUtils.getMd5(map);
			String encrypted = AppCommonUtils.encrypted(gson.toJson(map), commonKey, md5);
			Map<String,String> reqData = new HashMap<String, String>();
			reqData.put("action", "CC_QUERYPAY");
			reqData.put("data",encrypted);
			reqData.put("md5",md5);
			reqData.put("merchant_id",merId);
			reqData.put("version","2.0");
			String post_string = gson.toJson(reqData);
			String rspData_str =  HttpClient.jsonPost(post_string,requestBackUrl);

			Map<String, String> rspData = gson.fromJson(rspData_str, Map.class);
			String rsp_code = rspData.get("rsp_code");
			if ("SUCCESS".equalsIgnoreCase(rsp_code)) {
				String data = rspData.get("data");
				String md5_rtn = rspData.get("md5");
				String decipher = AppCommonUtils.decipher(data, commonKey, md5_rtn);
				LogUtil.writeLog("decipher :"+ decipher);
				resp.getWriter().write("<!doctype html><body>"+"Request message:<br/>"+gson.toJson(map)+"<br/>" + "Response message:</br>"+decipher+"</body></html>");
			}else{
				resp.getWriter().write("<!doctype html><body>"+"Request message:<br/>"+gson.toJson(map)+"<br/>" + "Response message: Error "+rspData.get("rsp_msg")+"</br>"+"</body></html>");
			}
			/**To proceed the response codes, you need to compile a program based on your business logics. The logics for response code processing below are for reference only.------------->**/
			//For response code specifications, refer to Part 5 “Appendix” in Specifications for Platform Access Interfaces- by selecting Help Center > Download > Product Interface Specifications at open.unionpay.com.
			String reqMessage = DemoBase.genHtmlResult(reqData);
			String rspMessage = DemoBase.genHtmlResult(rspData);
			resp.getWriter().write("Request message:<br/>"+reqMessage+"<br/>" + "Response message:</br>"+rspMessage+"");
		}catch(Exception ex) {
			LogUtil.writeErrorLog("Error in card purchase: "+ex.getMessage());
			resp.getWriter().write("Request message:<br/>"+gson.toJson(map)+"<br/>" + "Response message:</br>Error: "+ex.getMessage());
		}
	}
}
